<?php
/**
* @Copyright Copyright (C) 2019 jtotal.org. All rights reserved.
* @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
* @link     	https://jtotal.org
*
* @copyright   Copyright (C) 2005 - 2019 jtotal.org | Open Source Matters, Inc. All rights reserved.
* @license     GNU General Public License version 2 or later; see LICENSE.txt
*/

use Joomla\CMS\Access\Access;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Factory;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

// no direct access
defined('_JEXEC') or die;

$com_path = JPATH_SITE.'/components/com_content/';
require_once $com_path.'router.php';
require_once $com_path.'helpers/route.php';

//jimport('joomla.application.component.model');
//JModel::addIncludePath($com_path.DS.'models');
JModelLegacy::addIncludePath($com_path . '/models', 'ContentModel');

abstract class ModVisualMenuHelper{

	public static $debug_msg = array();

	public static $quality = 80;
	public static $generate_thumbs = 0;
	public static $introtext_remove = '';


	public static function getList(&$params)
	{

		$jinput = Factory::getApplication()->input;

		//PARAMS
		self::$quality =  $params->get('thumb_quality', 80);
		self::$generate_thumbs =  $params->get('generate_thumbs', 0);
		self::$introtext_remove =  $params->get('introtext_remove', '');

		



		// Get an instance of the generic articles model
		$articles = JModelLegacy::getInstance('Articles', 'ContentModel', array('ignore_request' => true));

		//Set application parameters in model
		$app = Factory::getApplication();
		$appParams = $app->getParams();
		$articles->setState('params', $appParams);

		// Set the filters based on the module params
		$articles->setState('list.start', 0);
		$articles->setState('list.limit', (int) $params->get('count', 0));
		$articles->setState('filter.published', 1);

		// Access filter
		$access = !ComponentHelper::getParams('com_content')->get('show_noauth');
		$authorised = Access::getAuthorisedViewLevels(Factory::getUser()->get('id'));
		$articles->setState('filter.access', $access);

		// Prep for Normal or Dynamic Modes
		$mode = $params->get('mode', 'normal');
		switch ($mode)
		{
			case 'automatic':
				$option = $jinput->get('option');
				$view = $jinput->get('view');

				if ($option === 'com_content') {
					switch($view){
						case 'category':
							$catids = array($app->input->getInt('id'));
							
							break;
						case 'categories':
							$catids = array($app->input->getInt('id'));
							
							break;
						case 'article':
							if ($params->get('show_on_article_page', 1)) {

								$article_id = $app->input->getInt('id');
								$catid = $app->input->getInt('catid');

								if (!$catid){
									// Get an instance of the generic article model
									$article = JModelLegacy::getInstance('Article', 'ContentModel', array('ignore_request' => true));

									$article->setState('params', $appParams);
									$article->setState('filter.published', 1);
									$article->setState('article.id', (int) $article_id);
									$item = $article->getItem();

									$catids = array($item->catid);
								}
								else
								{
									$catids = array($catid);
								}
							}
							else {
								// Return right away if show_on_article_page option is off
								return;
							}
							break;

						case 'featured':
						default:
							// Return right away if not on the category or article views
							return;
					}
				}else {
					// Return right away if not on a com_content page
					return;
				}
				break;
			case 'normal':
			default:
				$catids = $params->get('catid');
				$articles->setState('filter.category_id.include', (bool) $params->get('category_filtering_type', 1));
				break;
		}


		// Category filter
		if ($catids) {

			//si tiene hijos y el level es mayor q 0
			if ($params->get('show_child_category_articles', 0) && (int) $params->get('levels', 0) > 0) {
		        // Get an instance of the generic categories model

		        $categories = JModelLegacy::getInstance('Categories', 'ContentModel', array('ignore_request' => true));
		        $categories->setState('params', $appParams);
		        $levels = $params->get('levels', 1) ? $params->get('levels', 1) : 9999;

		        //echo "tiene hijos | level:".$levels." | catids: ".$catids[0]." | count:".count($catids)."<br>";

		        $categories->setState('filter.get_children', $levels);
		        $categories->setState('filter.published', 1);
		        $categories->setState('filter.access', $access);
		        $additional_catids = array();

		        //echo "categorias hijo: ";

		        foreach($catids as $catid){

		            $categories->setState('filter.parentId', $catid);
		            $recursive = true;
		            $items = $categories->getItems($recursive);

		            if ($items){
			            foreach($items as $category)
			            {
			            	$condition = (($category->level - $categories->getParent()->level) <= $levels);
			            	if ($condition) {
								$additional_catids[] = $category->id;
								//echo $category->id."  |   ";
			            	}
			            }
			        }
		        }

		         /****** HIDE CATEGORIES ******/
				$hide_current_cat = $params->get('hide_current', '1');


		        if($hide_current_cat=="0"){
			        if(count($additional_catids)==0){
				        $catids = array_unique(array_merge($catids, $additional_catids));
				    }else{
				    	$catids = $additional_catids;
				    }
				}else{
					$catids = array_unique(array_merge($catids, $additional_catids));
				}

		    }


		   /******MULTICAT******/
		   //Valoro qué categoría debería aparecer en caso de que haya multiples
		  	$cat_itemid_based = $params->get('cat_itemid_based', '0');
		   
		  	if($cat_itemid_based){
		  		

		  		if(is_array($catids)){


		  			$current_itemid = $jinput->get('Itemid');
		  			
		  			$multi_menu = $app->getMenu();
		  			$multi_item = $multi_menu->getItem($current_itemid);
		  			$multi_link = new URI($multi_item->link);
		  			$multi_view = $multi_link->getVar('view');
		  			$multi_id = $multi_link->getVar('id');

		  			//es category
		  			if($multi_view=='category'){
		  				$articles->setState('filter.category_id', $multi_id);
		  			//es article	
		  			}elseif($multi_view=='article'){


		  				//get category id
		  				$db = Factory::getDBO();
		  				$db->setQuery('SELECT cat.id FROM #__categories cat RIGHT JOIN #__content cont ON cat.id = cont.catid WHERE cont.id='.$multi_id);   
     					$category_title = $db->loadResult();
						$articles->setState('filter.category_id', $category_title);

		  				//$articles->setState('filter.category_id', $catids);
		  			//es cualquier cosa
		  			}else{
		  				$articles->setState('filter.category_id', $catids);
		  			}
		  		}else{
		  			$articles->setState('filter.category_id', $catids);
		  		}
		  	}else{
		  		$articles->setState('filter.category_id', $catids);
		  	}



			
		}

		//Feature filter

		//only if is last category
		$showfeatures = $params->get('show_front', 'show');

		if($showfeatures=="hide_last_cat"){
			
			//compruebo si es estoy en el último nivel
	        if(count($additional_catids)==0 ){
	        	//echo "<br>Es el ultimo nivel!!!! ". count($additional_catids)."<br>";
	        	$articles->setState('filter.featured','hide');
	        }else{
				//echo "<br>NO es el ultimo nivel!!!! ". count($additional_catids)."<br>";
				$articles->setState('filter.featured', 'only');
	        }
	    }else{
	    	$articles->setState('filter.featured', $params->get('show_front', 'show'));
	    }

	    $usuario = $params->get('author_filtering', "");
	    $usuario_include = $params->get('author_filtering_type', "");
		
		// Ordering
		$articles->setState('list.ordering', $params->get('article_ordering', 'a.ordering'));
		$articles->setState('list.direction', $params->get('article_ordering_direction', 'ASC'));

		// New Parameters
		//$articles->setState('filter.featured', $params->get('show_front', 'show'));
		$articles->setState('filter.author_id', $params->get('author_filtering', ""));
		$articles->setState('filter.author_id.include', $params->get('author_filtering_type', 1));
		$articles->setState('filter.author_alias', $params->get('created_by_alias', ""));
		$articles->setState('filter.author_alias.include', $params->get('author_alias_filtering_type', 1));
		$excluded_articles = $params->get('excluded_articles', '');

		if ($excluded_articles) {
			$excluded_articles = explode("\r\n", $excluded_articles);
			$articles->setState('filter.article_id', $excluded_articles);
			$articles->setState('filter.article_id.include', false); // Exclude
		}

		$date_filtering = $params->get('date_filtering', 'off');
		if ($date_filtering !== 'off') {
			$articles->setState('filter.date_filtering', $date_filtering);
			$articles->setState('filter.date_field', $params->get('date_field', 'a.created'));
			$articles->setState('filter.start_date_range', $params->get('start_date_range', '1000-01-01 00:00:00'));
			$articles->setState('filter.end_date_range', $params->get('end_date_range', '9999-12-31 23:59:59'));
			$articles->setState('filter.relative_date', $params->get('relative_date', 30));
		}

		// Filter by language
			$articles->setState('filter.language',$app->getLanguageFilter());
			$items = $articles->getItems();
			

		// Display options
			$show_date = $params->get('show_date', 0);
			$show_date_field = $params->get('show_date_field', 'created');
			$show_date_format = $params->get('show_date_format', 'Y-m-d H:i:s');
			$show_category = $params->get('show_category', 0);
			$show_hits = $params->get('show_hits', 0);
			$show_author = $params->get('show_author', 0);
			$show_introtext = $params->get('show_introtext', 0);
			$introtext_limit = $params->get('introtext_limit', 100);

		//Thumbs

		self::$generate_thumbs = $params->get('generate_thumbs', 0);
		$thumbs_path = $params->get('thumbs_path', 'dx_visualmenu_thumbs');
		$thumb_width = $params->get('thumb_width', 100);

		$webp = $params->get('vm_support_webp', '0');
		$tech = $params->get('vm_generate_tech', '0');


		if(!self::_checkGD()) self::$generate_thumbs = 0;

		// Find current Article ID if on an article page
		$option = $jinput->get('option');
		$view = $jinput->get('view');

		if ($option === 'com_content' && $view === 'article') {
			$active_article_id = $jinput->getInt('id');
		}
		else {
			$active_article_id = 0;
		}

		/****** CATEGORIES AS ITEMS ******/
		$cats_as_items = $params->get('category_as_item', '0');
		if($cats_as_items=='1'){
			$cat_ids = join(',',$catids);
			
			$db = Factory::getDbo();
			$query = $db->getQuery(true);
			$query = "SELECT * from #__categories WHERE parent_id IN($cat_ids) AND published = 1";
			$db->setQuery($query);
			$row = $db->loadAssocList();

			//var_dump($articles);

			foreach ($row as $key => $cat){
				//var_dump($row[$key]['params']);
				
				$catList = new JObject;
				$catList->set('iscat', '1');
				$catList->set('id', $row[$key]['id']);
				$catList->set('title', $row[$key]['title']);
				$catList->set('alias', $row[$key]['alias']);
				$catList->set('catid', $row[$key]['id']);
				$catList->set('images', $row[$key]['params']);
				$catList->set('created', $row[$key]['created_time']);
				//$catList->setState('list.ordering', $params->get('article_ordering', 'a.ordering'));
				//$catList->setState('list.direction', $params->get('article_ordering_direction', 'ASC'));

				array_push($items, $catList);
			
			}
		}


		$forceitemID = $params->get('force_itemid', '0');
		// Prepare data for display using display options

		$current_itemid = $jinput->getInt('Itemid');

		foreach ($items as &$item){
			$item->slug = $item->id.':'.$item->alias;
			//$item->catslug = $item->catid ? $item->catid .':'.$item->category_alias : $item->catid;



			if ($access || in_array($item->access, $authorised)){
				// We know that user has the privilege to view the article
				
				//var_dump($item);

				if(isset($item->iscat)){

					//category
					/*$url = ContentHelperRoute::getCategoryRoute($item->id);
					$url = str_replace("&layout=","",$url);
					$itemid_pieces = explode('&', $url);
					$cuenta = count($itemid_pieces);
					$numbers_itemid = explode('=', $itemid_pieces[$cuenta-1]);
					$itemid = $numbers_itemid[1];
 

					$application = JFactory::getApplication();
					$menu = JFactory::getApplication()->getMenu();
					$item_yeah = $menu->getItem('21');
				


					var_dump($item->id);
					var_dump($url);
			
					$link_yeah = new JURI($item_yeah->link_yeah);
*/
					//OLD
					//$item->link = JRoute::_($item_yeah->link.'&Itemid='.$itemid);
					
					//from 4.2.0
					$item->link = Route::_('index.php?option=com_content&view=category&id='.$item->id);



				}else{

					$item->catslug = $item->catid ? $item->catid .':'.$item->category_alias : $item->catid;

					//article
					//forceID
					if($forceitemID==1){
						$item->link = Route::_('index.php?option=com_content&view=article&id='.$item->slug.'&catid='.$item->catslug.'&Itemid='.$current_itemid);
					}else{
						$item->link = Route::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catslug));
					}
					
				}
			} else {
				// Angie Fixed Routing
				$app	= Factory::getApplication();
			    $menu	= $app->getMenu();
				$menuitems	= $menu->getItems('link', 'index.php?option=com_users&view=login');
				
				if(isset($menuitems[0])) {
					$Itemid = $menuitems[0]->id;
				} else if ($jinput->getInt('Itemid') > 0) { //use Itemid from requesting page only if there is no existing menu
					$Itemid = $jinput->getInt('Itemid');
				}

				$item->link = Route::_('index.php?option=com_users&view=login&Itemid='.$Itemid);
			}

			// Used for styling the active article
			$item->active = $item->id == $active_article_id ? 'active' : '';

			$item->displayDate = '';
			if ($show_date) {
				
				$date = new Date($item->$show_date_field);
				$item->displayDate= $date->format($show_date_format);
			}

			if ($item->catid) {
				$item->displayCategoryLink = Route::_(ContentHelperRoute::getCategoryRoute($item->catid));
				$item->displayCategoryTitle = $show_category ? '<a href="'.$item->displayCategoryLink.'">'.$item->category_title.'</a>' : '';
			}
			else {
				$item->displayCategoryTitle = $show_category ? $item->category_title : '';
			}

			$item->displayHits = $show_hits ? $item->hits : '';
			$item->displayAuthorName = $show_author ? $item->author : '';
			

			/**********************************/
			/********** IMAGES ****************/
			/**********************************/
			
			//creo el directorio si no esta creado
			$ruta_thumb = JPATH_BASE.'/cache/'.$thumbs_path;
			
			//var_dump($ruta);
			self::_createPath($ruta_thumb);

			//busco imagen en la configuracion del article (intro image)
			$introimage = json_decode($item->images);

			$no_imagen = false;
		
			//CHECK IMAGES in INTROIMAGE
			if(!isset($introimage->image_intro)){
				$no_imagen = true;
			}else{
			
		
				if($introimage && strlen($introimage->image_intro)!=0){

					if(self::$generate_thumbs){
						$item->displayImage = self::_createThumbs($introimage->image_intro,$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);
					}else{
						$item->displayImage = $introimage->image_intro;
					}

					//$item->displayImage = $introimage->image_intro;
					$item->displayAlt = $introimage->image_intro_alt;

				}elseif($introimage && strlen($introimage->image_fulltext)!=0){

					//self::_createPath();
					//self::$generate_thumbs = $params->get('generate_thumbs', 1);
					//$thumbs_path = $params->get('thumbs_path', 'images/dx_visualmenu_thumbs');
					//$thumb_width = $params->get('thumb_width', 100);

					if(self::$generate_thumbs){
						$item->displayImage = self::_createThumbs($introimage->image_fulltext,$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);
					}else{
						$item->displayImage = $introimage->image_fulltext;
					}

					$item->displayAlt = $introimage->image_fulltext_alt;
		

				}elseif($introimage && isset($introimage->image)){

					//cat image
					if(self::$generate_thumbs){
						$item->displayImage = self::_createThumbs($introimage->image,$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);

					}else{
						$item->displayImage = $introimage->image;
					}
					//$item->displayImage = $introimage->image;
					$item->displayAlt = $introimage->image;
				}else{
					$no_imagen = true;
				}
			}

			if(!isset($item->introtext)) $item->introtext = '';

			if($no_imagen){
				
				$item->introtext_prepared = JHtml::_('content.prepare', $item->introtext);
				
		
				$imagen = preg_match_all('/<img[^>]+>/i', $item->introtext_prepared, $match);

				//si hay imagenes
				/*TODO: comprobar si hay intro-image en el artículo*/

			
				if((bool) $imagen){
					//consigo el alt
					$src = preg_match_all('/(alt|title|src)=("[^"]*")/i', $match[0][0], $match_src);
					
					//compruebo la posición del src en el array
					if($match_src[1][0]=='src'){
						$ruta = $match_src[2][0];
						$alt = $match_src[2][1];
					}else{
						$ruta = $match_src[2][1];
						$alt = $match_src[2][0];
					}
			
					//$imgEPos = JString::strpos($item->introtext,'"',$imgSPos+5);
					//$imgEAlt = JString::strpos($item->introtext,'"',$imgSAlt+5);
					//echo "cosa:".htmlentities(JString::substr($item->introtext, ($imgSPos+5), ($imgEPos-($imgSPos+5))));
					//Si hay más de una imagen, inserto sólo la primera
					/*if($imgSPos > 0){
						$item->displayImage = JString::trim($ruta, '"');
						$item->displayAlt = JString::substr($item->introtext, ($imgSAlt+5), ($imgEAlt-($imgSAlt+5)));
					}*/
					//$item->displayImage = JString::trim($ruta, '"');
					if(self::$generate_thumbs){
						$item->displayImage = self::_createThumbs(JString::trim($ruta, '"'),$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);


					}else{
						$item->displayImage = JString::trim($ruta, '"');
					}

					$item->displayAlt = JString::trim($alt, '"');
				} 
				//$item->introtext =  self::_cleanIntrotext($item->introtext,$params->get('introtext_max', 200));
				//$item->displayIntrotext = $show_introtext ? self::truncate($item->introtext, $introtext_limit) : '';
				// added Angie show_unauthorized

				if(!isset($item->alternative_readmore)) $item->alternative_readmore = '';
				$item->displayReadmore = $item->alternative_readmore;
			}

			//MODIFIERS
			if($show_introtext){

				


				$introtext_modified =  self::_cleanIntrotext($item->introtext);
				$item->displayIntrotext = JHtmlString::truncateComplex($introtext_modified,(int)$params->get('introtext_max', 200, 'INT'));

				//var_dump($introtext_modified);
			}
			//var_dump($item->displayIntrotext);


		}
		
		//order articles with categories with date
		
		if(isset($item->iscat)){
			$items = self::order_cat_items($items);
		}



		return $items;
	}



	/*********** API */
	/*********** API */
	/*********** API */
	/*********** API */
	/*********** API */
	public static function getApiList(&$params){

		self::$generate_thumbs =  $params->get('generate_thumbs', 0);
		self::$introtext_remove =  $params->get('introtext_remove', '');


		$model = JModelLegacy::getInstance('Articles', 'ContentModel', array('ignore_request' => true));

		//PARAMS
		$api_url = $params->get('api_url','');

		if($api_url == ''){
			self::addDebug("getApiList","Empty api");
			return;
		}

		//FEATURED
		$featured_param = $params->get('show_front', '');

		if ($featured_param === ''){$featured = 'show';
		}elseif ($featured_param){$featured = 'only';
		}else{$featured = 'hide';}

		//CATS
		$cats = $params->get('cat_id', '');

		//LIMIT
		$total_articles = $params->get('count', 0);
		if($total_articles == 0) $total_articles = 5;

		//DIRECTION
		$direction = $params->get('article_ordering_direction', 'DESC');


		//CALL HOME
		/*
		$final_url = $api_url.'/index.php?option=com_api&app=jtarticles&resource=latest
		&featured='.$featured.'
		&catid='.$cats.'
		&limit='.$total_articles.'
		&direction='.$direction.'
		&tag='.$tags_comma.'
		&format=json';*/
		$final_url = '';
		$final_url .= $api_url.'/index.php?option=com_api&app=jtarticles&resource=latest';
		$final_url .= '&featured='.$featured;
		$final_url .= '&catid='.$cats;
		$final_url .= '&limit='.$total_articles;
		$final_url .= '&direction='.$direction;
		$final_url .= '&format=json';


		//CLEAN
		$final_url = str_replace(array("\r\n", "\r", "\t", "\n", " "), "", $final_url);
		$final_url = OutputFilter::ampReplace(htmlspecialchars($final_url, ENT_COMPAT, 'UTF-8', false));
		self::addDebug("getApiList","api url: ".$final_url);
		$call_url = file_get_contents($final_url);
		$result = json_decode($call_url);

		$items = $result->data->data;

		/********** IMAGES PARAMS****************/
		//Thumbs
		$thumbs_path = $params->get('thumbs_path', 'dx_visualmenu_thumbs');
		$thumb_width = $params->get('thumb_width', 100);
		if(!self::_checkGD()) self::$generate_thumbs = 0;
		
		
		$webp = $params->get('vm_support_webp', '0');
		$tech = $params->get('vm_generate_tech', '0');

		//creo el directorio si no esta creado
		$ruta_thumb = JPATH_BASE.'/cache/'.$thumbs_path;
		self::_createPath($ruta_thumb);



		//MODIFY THINGS!
		foreach ($items as &$item){

			//MODIFIERS

			$item->active = '';

			//introtext
			if($params->get('show_introtext', 0)){
				$introtext_modified =  self::_cleanIntrotext($item->introtext);
				$item->displayIntrotext = JHtmlString::truncateComplex($introtext_modified,(int)$params->get('introtext_max', 200, 'INT'));
			}

			/**********************************/
			/********** IMAGES ****************/
			/**********************************/
	
			$introimage = $item->images;

			if($introimage && strlen($introimage->image_intro)!=0){

				

				if(self::$generate_thumbs){
				
					$item->displayImage = self::_createThumbs($introimage->image_intro,$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);
					

				}else{
					$item->displayImage = $introimage->image_intro;
				}

				//$item->displayImage = $introimage->image_intro;
				$item->displayAlt = $introimage->image_intro_alt;

			}elseif($introimage && strlen($introimage->image_fulltext)!=0){

				//self::_createPath();
				//self::$generate_thumbs = $params->get('generate_thumbs', 1);
				//$thumbs_path = $params->get('thumbs_path', 'images/dx_visualmenu_thumbs');
				//$thumb_width = $params->get('thumb_width', 100);

				if(self::$generate_thumbs){
					$item->displayImage = self::_createThumbs($introimage->image_fulltext,$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);
				}else{
					$item->displayImage = $introimage->image_fulltext;
				}

				$item->displayAlt = $introimage->image_fulltext_alt;
	

			}elseif($introimage && isset($introimage->image)){

				//cat image
				if(self::$generate_thumbs){
					$item->displayImage = self::_createThumbs($introimage->image,$ruta_thumb,$thumbs_path,$thumb_width,$webp,$tech);

				}else{
					$item->displayImage = $introimage->image;
				}
				//$item->displayImage = $introimage->image;
				$item->displayAlt = $introimage->image;
			}


			
			


		}

	

	



		return $items;
	}





	/*create thumbs*/
	public static function _createThumbs($introimage_thumb,$ruta_thumb,$thumbs_path,$thumb_width, $webp=0,$tech=0) {
		$resultado = array();
		
		//get extension
		$ext_boom  = explode('.', $introimage_thumb);
		$ext = strtolower(end($ext_boom));


		if(self::$generate_thumbs){
			
			//external or internal image
			if(self::check_http_img($introimage_thumb)){
				$ruta_img = $introimage_thumb;
			}else{
				$ruta_img = JPATH_BASE.'/'.$introimage_thumb;
			}
			
			//codification 76b61fb94a1f03dc7382c67c69537f54bd4ef4e9
			$thumb_name = sha1($introimage_thumb);

			//if webp support is activated
			$existe = false;
			if (file_exists($ruta_thumb.'/'.$thumb_name.'.'.$ext)) {

				$resultado['path'] = 'cache/'.$thumbs_path.'/'.$thumb_name;
				$resultado['ext'] = $ext;
				$resultado['webp'] = false;
	
				if($webp){
					if (file_exists($ruta_thumb.'/'.$thumb_name.'.webp')) {
						$resultado['webp'] = true;
					}
				}
				$existe = true;
			}




			//CHECK and return (HORAY!!) or GENERATE
			if ($existe) {
				self::addDebug("Generate thumb: ",$thumb_name.' exists. Avoid thumb generation');
				return $resultado;
			}else{
				self::addDebug("Generate thumb: ",$introimage_thumb.' doesnt exist. Generating');
				
				//Imagick
				if($tech){
					if(class_exists("Imagick")){
					   	self::_createThumb_imagick($ext,$ruta_img,$ruta_thumb, $thumb_name,$thumb_width,$webp);
					//fallback
					}else{
						self::_createThumb_raw($ruta_img,$ruta_thumb,$thumb_name,$thumb_width,false, $webp);
					}
				}else{
					self::_createThumb_raw($ruta_img,$ruta_thumb,$thumb_name,$thumb_width,false, $webp);
				}

			}

			//RECHECK
			if (file_exists($ruta_thumb.'/'.$thumb_name.'.'.$ext)) {
				$resultado['path'] = 'cache/'.$thumbs_path.'/'.$thumb_name;
				$resultado['ext'] = $ext;
				$resultado['webp'] = false;
	
				if($webp){
					if (file_exists($ruta_thumb.'/'.$thumb_name.'.webp')) {
						$resultado['webp'] = true;
					}
				}
				$existe = true;
			}

			//CHECK and return (HORAY!!) or... buuuuh!
			if ($existe) {
				self::addDebug("Generate thumb (recheck): ",$thumb_name.' exists in recheck');
				return $resultado;
			}else{
				$resultado = false;
			}

		}

		return $resultado;
	}


	/*create path for thumbs*/
	public static function _checkGD() {
		if(!extension_loaded('gd')){
			return false;
		}else{
			return true;
		}
	}

	/*create path for thumbs*/
	public static function _createPath($path) {
	    if (is_dir($path)) return true;
	    $prev_path = substr($path, 0, strrpos($path, '/', -2) + 1 );
	    $return = self::_createPath($prev_path);
	    return ($return && is_writable($prev_path)) ? mkdir($path) : false;
	}

	public static function _createThumb_imagick($ext,$ruta_img,$ruta_thumb,$filename,$imgwidth,$webp){
			self::addDebug("_createThumb_imagick",$ruta_img.' | '.$imgwidth);
		    $img = new Imagick($ruta_img); 
		    $img->setImageResolution(72,72); 
		    $img->resampleImage(72,72,imagick::FILTER_UNDEFINED,1); 
		    $img->scaleImage($imgwidth,0); 
		    $img->writeImage($ruta_thumb.'/'.$filename.'.'.$ext); 
			self::addDebug("_createThumb_imagick: writeImage",$filename.' | '.$ext);
			
		    if($webp){
				$img->writeImage($ruta_thumb.'/'.$filename.'.webp'); 
				self::addDebug("_createThumb_imagick: writeImage",$filename.' | webp');
		    }
		    
		    $img->destroy(); 
	}


	public static function _createThumb_raw($src,$dest,$thumb_name,$desired_width = false, $desired_height = false, $webp){

		self::addDebug("_createThumb_RAW",$src.' | '.$dest.' | '.$thumb_name.' | '.$desired_width.' | '.$desired_height.' | '.$webp);
		
		$quality = self::$quality;

		//CHECK IF IMAGEWEB creation is available
		if($webp){
			if(function_exists('imagewebp') === false){
				$webp = 0;
			}
		}

	    /*If no dimenstion for thumbnail given, return false */
	    if (!$desired_height&&!$desired_width){
			self::addDebug("RETURN _createThumb_RAW",$desired_height.' | '.$desired_width.' | !$desired_height&&!$desired_width');
			return false;
		}
	    $fparts = pathinfo($src);
	    $ext = strtolower($fparts['extension']);
	    /* if its not an image return false */
	    if (!in_array($ext,array('gif','jpg','png','jpeg'))){
			self::addDebug("RETURN _createThumb_RAW",$ext.' |  !in_array($ext,array(gif,jpg,png,jpeg)');
			return false;
		}

		if(self::check_http_img($src) == false){
			
			if(!file_exists($src)){
				self::addDebug("RETURN _createThumb_RAW",$src.' |  file exists?)');
				return false;
			}
		}
	   

	    /* read the source image */
	    if ($ext == 'gif'){
	        $resource = imagecreatefromgif($src);
		}else if ($ext == 'png'){
	        $resource = imagecreatefrompng($src);
	    }else if ($ext == 'jpg' || $ext == 'jpeg'){
	        $resource = imagecreatefromjpeg($src);
	    }
	    $width  = imagesx($resource);
	    $height = imagesy($resource);

	    /* find the "desired height" or "desired width" of this thumbnail, relative to each other, if one of them is not given  */
	    if(!$desired_height) $desired_height = floor($height*($desired_width/$width));
	    if(!$desired_width)  $desired_width  = floor($width*($desired_height/$height));
	  
	    /* create a new, "virtual" image */
	    $virtual_image = imagecreatetruecolor($desired_width,$desired_height);



		switch ($ext)
		{
		    case 'jpg':
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        break;
		    case 'jpeg':
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        break;

		    case 'png':
		        imagealphablending($virtual_image, FALSE);
		        imagesavealpha($virtual_image, TRUE);
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        break;
		    case 'gif':
		        imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);
		        $background = imagecolorallocate($virtual_image, 0, 0, 0); 
		        imagecolortransparent($virtual_image, $background);
		        break;
		}

	    //better quality
	    //old
	    //imagecopyresampled($virtual_image,$resource,0,0,0,0,$desired_width,$desired_height,$width,$height);

	    $fparts = pathinfo($dest.'/'.$thumb_name);

	    //$ext = strtolower($fparts['extension']);

	    /* if dest is not an image type, default to jpg */
	    if (!in_array($ext,array('gif','jpg','png','jpeg'))) $ext = 'jpg';
	    $dest = $fparts['dirname'].'/'.$fparts['filename'].'.'.$ext;

	    //webp
	    $dest_webp = $fparts['dirname'].'/'.$fparts['filename'].'.webp';
	    
	    if ($ext == 'gif'){
	        imagegif($virtual_image,$dest);
	    }else if ($ext == 'png'){
	        imagepng($virtual_image,$dest,8);
	        if($webp) imagewebp($virtual_image,$dest_webp,$quality);
	    }else if ($ext == 'jpg' || $ext == 'jpeg'){
	        imagejpeg($virtual_image,$dest,$quality);
			if($webp) imagewebp($virtual_image,$dest_webp,$quality);
			//self::addDebug("raw: imagewebp",$dest_webp);
	    }
		self::addDebug("_createThumb_raw: writeImage",'image created: '.$dest.' | q: '.$quality);
	    return array(
	        'width'     => $width,
	        'height'    => $height,
	        'new_width' => $desired_width,
	        'new_height'=> $desired_height,
	        'dest'      => $dest,
	        'dest_webp' => $dest_webp
	    );
	}
	/**********************************/
	/******* fin de IMAGES ************/
	/**********************************/


	public static function _cleanIntrotext($introtext,$max = 200)
	{
		$introtext = str_replace('<p>', ' ', $introtext);
		$introtext = str_replace('</p>', ' ', $introtext);
		$introtext = strip_tags($introtext,'<a><strong>');
		//$introtext = strip_tags($introtext);
		$introtext = trim($introtext);


		//custom remove from introtext
		if(self::$introtext_remove != ''){
			$pieces = array_map('trim', explode(',', self::$introtext_remove));
			
			foreach($pieces as $key => $piece){
				$regex		= '/'.$piece.'(.*?)}/i';
				preg_match_all($regex, $introtext, $matches, PREG_SET_ORDER);
	
				foreach ($matches as $match){
					$introtext = str_replace($match[0], '', $introtext);
					self::addDebug("_cleanIntrotext: loadmoduleid",$match[0].' REMOVED');
				}
			}
		}
		
		return $introtext;
	}


	//function to order categories and items by date

	public static function order_cat_items($items){
		//var_dump($items[0]->created);

		$ord = array();
		foreach ($items as $key => $item){
		    $ord[] = strtotime($item->created);
		    //var_dump($item->created);
		}
		array_multisort($ord, SORT_ASC, $items);
		return $items;
	}

	public static function addDebug($func='', $msg='')
	{
		$cuenta = count(self::$debug_msg);
		self::$debug_msg[$cuenta]['func'] = $func;
		self::$debug_msg[$cuenta]['msg'] = $msg;
	}

	public static function getDebug()
	{
		return self::$debug_msg;
	}


	public static function check_http_img($introimage_thumb){

		if(substr( $introimage_thumb, 0, 4 ) === "http"){
			return true;
		}else{
			return false;
		}

	}

	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/********************OLD ********************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */
	/**************************************************** */

	public static function truncate($html, $maxLength = 200)
	{
	    $printedLength = 0;
	    $position = 0;
	    $tags = array();

	    $output = '';

	    if (empty($html)) {
			return $output;
	    }

	    while ($printedLength < $maxLength && preg_match('{</?([a-z]+)[^>]*>|&#?[a-zA-Z0-9]+;}', $html, $match, PREG_OFFSET_CAPTURE, $position))
	    {
	        list($tag, $tagPosition) = $match[0];

	        // Print text leading up to the tag.
			$str = JString::substr($html, $position, $tagPosition - $position);
	        if ($printedLength + JString::strlen($str) > $maxLength) {
	            $output .= JString::substr($str, 0, $maxLength - $printedLength);
	            $printedLength = $maxLength;
	            break;
	        }

	        $output .= $str;
	        $lastCharacterIsOpenBracket = (JString::substr($output, -1, 1) === '<');

	        if ($lastCharacterIsOpenBracket) {
				$output = JString::substr($output, 0, JString::strlen($output) - 1);
	        }

	        $printedLength += JString::strlen($str);

	        if ($tag[0] == '&') {
	            // Handle the entity.
	            $output .= $tag;
	            $printedLength++;
	        }
	        else {
	            // Handle the tag.
	            $tagName = $match[1][0];

	            if ($tag[1] == '/') {
	                // This is a closing tag.
	                $openingTag = array_pop($tags);

	                $output .= $tag;
	            }
	            else if ($tag[JString::strlen($tag) - 2] == '/') {
	                // Self-closing tag.
	                $output .= $tag;
	            }
	            else {
	                // Opening tag.
	                $output .= $tag;
	                $tags[] = $tagName;
	            }
	        }

	        // Continue after the tag.
	        if ($lastCharacterIsOpenBracket) {
				$position = ($tagPosition - 1) + JString::strlen($tag);
			}
			else {
				$position = $tagPosition + JString::strlen($tag);
			}

	    }

	    // Print any remaining text.
	    if ($printedLength < $maxLength && $position < JString::strlen($html)) {
			$output .= JString::substr($html, $position, $maxLength - $printedLength);
	    }

	    // Close any open tags.
	    while (!empty($tags))
	    {
			$output .= sprintf('</%s>', array_pop($tags));
	    }

	    $length = JString::strlen($output);
	    $lastChar = JString::substr($output, ($length - 1), 1);
	    $characterNumber = ord($lastChar);

	    if ($characterNumber === 194) {
			$output = JString::substr($output, 0, JString::strlen($output) - 1);
	    }

		$output = JString::rtrim($output);

	    return $output;
	}



}
